#include <GameEngine/EventManager.h>
#include <GameEngine/GameEvent.h>

//#include <GameEngine/Serialization/Torment.h>
#include <GameEngine/GameModules.h>

#include <fstream>
#include <map>

unsigned long hash(const std::string& str) {
	unsigned long hash = 0;
    int c;
	const char* strn = str.c_str();
    while (c = *strn++)
        hash = c + (hash << 6) + (hash << 16) - hash;
    return hash;
}

unsigned long hash(const char* str)
{
    unsigned long hash = 0;
    int c;
    while (c = *str++)
        hash = c + (hash << 6) + (hash << 16) - hash;
    return hash;
}

struct EventData {
	EventData(const std::string& aName, const std::string& aDesc = std::string()) :
        name_(aName), desc_(aDesc)
    {
    }

	const std::string name_;
	const std::string desc_;
};

struct EventRegistry::PrivateData {
	std::map<GameEvent::EventID, EventData> events_;
    std::map<unsigned long, GameEvent::EventID> nameIds_;
};

EventRegistry::EventRegistry()
{
	pData_ = new PrivateData();

	REGISTER_EVENT(GameEvent::E_INVALID,"Invalid eventID")

		///////////////////////////////////////////////////////////////////////////////////////////
		/**
		 * 1. SET EVENTS:
		 * 
		 * @send:
		 * If these events are send by a component, it directly wants to modify an attribute of other components,
		 * or cause a specific action of another component.
		 * 
		 * @listen:
		 * If a component listens to such an event, it has an attribut that will be modified by the event,
		 * or an action that will be executed by the event,
		 * or it just wants to be notified when such an attribute changes or action occures in another component.
		 * 
		 * @eventData: the event data includes the new value for the attribute to set (or to modify it by),
		 * or optional information for the action to be executed
		 * 
		 * @allowed by:
		 * Every component can send and/or listen to these events.
		 * 
		 * @check?:
		 * You should call checkEvent() before executeEvent().
		 */ 
		///////////////////////////////////////////////////////////////////////////////////////////
		REGISTER_EVENT(GameEvent::E_SET_TRANSFORMATION, "A transformation change event, containing the new transformation")
		REGISTER_EVENT(GameEvent::E_SET_ROTATION,			 "Set the rotation of the object in world space")
		REGISTER_EVENT(GameEvent::E_SET_TRANSLATION,		 "A translation change event, containing new global translation (Vec3fData)")
		REGISTER_EVENT(GameEvent::E_SET_SCALE,				 "Set the scale of the object")
		REGISTER_EVENT(GameEvent::E_TRANSLATE_LOCAL,		 "Translate Object along it's local axis (using a Vec3fData)")
		REGISTER_EVENT(GameEvent::E_TRANSLATE_GLOBAL,		 "Translate object along the world axis (using a Vec3fData)")
		REGISTER_EVENT(GameEvent::E_ROTATE_LOCAL,			 "Rotate object about it's local axis (using a Vec3fData)")
		REGISTER_EVENT(GameEvent::E_MORPH_TARGET,			 "Sets the model morpher of a Horde3D representation to the provided morph target position")
		REGISTER_EVENT(GameEvent::E_MORPH_TARGET_ANIM,		 "Animates a morph target")
		REGISTER_EVENT(GameEvent::E_PLAY_ANIM,				 "Play an animation")
		REGISTER_EVENT(GameEvent::E_STOP_ANIM,"")
		REGISTER_EVENT(GameEvent::E_PAUSE_ANIM,"")
		REGISTER_EVENT(GameEvent::E_RESUME_ANIM,"")
		REGISTER_EVENT(GameEvent::E_UPDATE_ANIM,	"Updates a running animation")
		REGISTER_EVENT(GameEvent::E_SETUP_ANIM,"Loads an animation on a model stage")
		REGISTER_EVENT(GameEvent::E_SET_ANIM_FRAME,"Sets the frame of a loaded animation")
		REGISTER_EVENT(GameEvent::E_ATTACH,				"Attaches a SceneNode to another SceneNode")
		REGISTER_EVENT(GameEvent::E_SET_NODE_PARENT,		"Sets a new Parent for a SceneNode")
		REGISTER_EVENT(GameEvent::E_SPEAK,				"Speak a sentence")
		REGISTER_EVENT(GameEvent::E_SET_VOICE,			"Change the TTS Voice of a TTSComponent")
		REGISTER_EVENT(GameEvent::E_ACTIVATE_CAM,			"Make the current entity the active camera if it is one")
		REGISTER_EVENT(GameEvent::E_PERFORM_ACTION,		"Perform an action")
		REGISTER_EVENT(GameEvent::E_GO_TO_ENTITY,			"Move close to another entity")
		REGISTER_EVENT(GameEvent::E_GO_TO_POSITION,		"Move to a position")
		REGISTER_EVENT(GameEvent::E_GO_TO_STOPPED,		"Entity has arrived at the desired location")
		REGISTER_EVENT(GameEvent::E_SET_PROPERTY,			"Sets a property	")
		REGISTER_EVENT(GameEvent::E_ADJUST_PROPERTY,		"Adjusts a property")
		REGISTER_EVENT(GameEvent::E_INTERACT,				"Causes an Entity to interact with another Entity.")
		REGISTER_EVENT(GameEvent::E_INTERACT_PARTNER,		"Informs an Entity that another Entity is interacting with it and causes reactions for the interaction")
		REGISTER_EVENT(GameEvent::E_SET_SOUND_GAIN,		"Sets the sound gain")
		REGISTER_EVENT(GameEvent::E_SET_SOUND_LOOP,		"Sets the sound to loop")
		REGISTER_EVENT(GameEvent::E_SET_SOUND_OFFSET,		"Sets the offset of a sound")
		REGISTER_EVENT(GameEvent::E_SET_SOUND_FILE,		"Sets a sound file, also starts playing it")
		REGISTER_EVENT(GameEvent::E_SET_SOUND_WITH_USER_DATA, "Sets a sound already providing the decoded data")
		REGISTER_EVENT(GameEvent::E_PLAY_SOUND,			"Plays or resumes the currently set sound")
		REGISTER_EVENT(GameEvent::E_PAUSE_SOUND,			"Pauses a currently playing sound")
		REGISTER_EVENT(GameEvent::E_STOP_SOUND,			"Stops a currently playing sound")
		REGISTER_EVENT(GameEvent::E_SET_PHONEMES_FILE,	"Sets a phonemes file")
		REGISTER_EVENT(GameEvent::E_SET_ENABLED,			"Enables components (wether the component should be rendered/updated or not)	")
		REGISTER_EVENT(GameEvent::E_PICKUP,				"Attaches an entity under the child scene node of another entity")
		REGISTER_EVENT(GameEvent::E_PUTDOWN,				"Detaches an entity")
		REGISTER_EVENT(GameEvent::E_SET_MOVE_ANIM,		"Change one of the move animations. @data = pair<string, string>: tag, name of animation")
		REGISTER_EVENT(GameEvent::E_SET_CROWD_TAG,		"Sets the current crowd sim tag of a particle, so it starts wandering around if the tag > 0")
		REGISTER_EVENT(GameEvent::E_SET_CROWD_IGNORE_FORCES, "The crowd particle will ignore forces if the particle is not moving")
		REGISTER_EVENT(GameEvent::IK_SOLVE,			"Computes the IK with the given data and applies it to the chain")
		REGISTER_EVENT(GameEvent::IK_CREATEIKANIM,		"Computes the IK with the given data, generates a keyframe animation of the chain transformation and stores the anim resource in data->result")
		REGISTER_EVENT(GameEvent::IK_PLAYIKANIM,		"Plays a previously generated IKAnim")
		REGISTER_EVENT(GameEvent::IK_GAZE,				"Computes and applies a gaze action to the designated model (agent)")
		REGISTER_EVENT(GameEvent::IK_SETPARAMI,		"Sets an IK parameter (IK_Param) of type integer")
		REGISTER_EVENT(GameEvent::IK_SETPARAMF,		"Sets an IK parameter (IK_Param) of type float")
		REGISTER_EVENT(GameEvent::GB_ON_OFF,			"activates/deactivates character's gaze behaviour")
		REGISTER_EVENT(GameEvent::GB_CONTROL_ON_OFF,	"activates/deactivates direct control of character via arrow keys")
		REGISTER_EVENT(GameEvent::GB_SEARCH_ON_OFF,	"activates/deactivates character's searching behaviour")
		REGISTER_EVENT(GameEvent::GB_WAVE_BACK,		"makes the npc react to a waving event")
		REGISTER_EVENT(GameEvent::GB_RESET,			"reset all collected data")
		REGISTER_EVENT(GameEvent::SP_START_SCRIPT,		"starts sn action script")
		REGISTER_EVENT(GameEvent::IB_EYE_FIXATED,		"informs about an eye fixation")
		REGISTER_EVENT(GameEvent::FACS_SET_EXPRESSION,"sets a facial expression /* deprecated */")
		REGISTER_EVENT(GameEvent::E_SEND_SOCKET_DATA,"sends data via the socket component")
		REGISTER_EVENT(GameEvent::E_SET_FACS,"sets a facial expression")

		REGISTER_EVENT(GameEvent::AG_ANIM_PLAY,			"Loads an animation and plays it back. Param: AgentAnimationData(const char*/int,SourceType,int,float,float,int,char*,char*), returns:  an int")
		REGISTER_EVENT(GameEvent::AG_ANIM_STOP,			"stops an animation's playback. Param: AgentAnimationData(int), returns: int")
		REGISTER_EVENT(GameEvent::AG_ANIM_SET_EXTENT,		"Sets an animation's spatial extent. Param: AgentAnimationData(int,float)")
		REGISTER_EVENT(GameEvent::AG_ANIM_SET_SPEED,		"Sets the animation's playback speed. Param: AgentAnimationData(int,float)")
		REGISTER_EVENT(GameEvent::AG_ANIM_SET_STROKES,	"Sets the unmber of strokes an animations should perform. Param: AgentAnimationData(int,int)")
		REGISTER_EVENT(GameEvent::AG_ANIM_CLEAR,			"Clears all animation stages, deleting all loaded animations. No data required")
		REGISTER_EVENT(GameEvent::AG_MOVEMENT,			"Performs a movement. Param: AgentMovementData(Vec3f/int,Type,const char*,const char*,float,bool), returns:  an int")
		REGISTER_EVENT(GameEvent::AG_MOVEMENT_SET_SPEED,	"Sets the default movement speed. Param: float")
		REGISTER_EVENT(GameEvent::AG_GAZE,				"Gazes towards the target. Param: AgentGazeData(Vec3f/int,float,float), returns: int")
		REGISTER_EVENT(GameEvent::AG_GAZE_SET_SPEED,		"Sets the agent's default gaze speed. Param: float")
		REGISTER_EVENT(GameEvent::AG_HEADNOD,				"Starts an alternating head motion. Param: AgentGazeData(int,float,int,float,float)")
		REGISTER_EVENT(GameEvent::AG_HEADSHAKE,			"Starts an alternating head motion. Param: AgentGazeData(int,float,int,float,float)")
		REGISTER_EVENT(GameEvent::AG_FORMATION_ADD,		"Adds a member to the formation. Param: int")
		REGISTER_EVENT(GameEvent::AG_FORMATION_DEL,		"Removes a member from the formation. Param: int")
		REGISTER_EVENT(GameEvent::AG_FORMATION_EVENT,		"Fires an event meant to attract the attention of the members of the formation. Param: int")
		REGISTER_EVENT(GameEvent::AG_FORMATION_REACT,		"Reacts to the specified event. Param: AgentFormationData(int,int)")
		REGISTER_EVENT(GameEvent::AG_SET_IPDIST,			"Sets the interpersonal distance constraints of the agent. Param: AgentFormationData(float,float)")
		REGISTER_EVENT(GameEvent::AG_SET_ORIENTCUST,		"Sets the deviation from the normal orientation of the agent. Param: float")
		REGISTER_EVENT(GameEvent::AG_SET_REPOSANIM,		"Sets the  repositioning animation name. Param: const char*")
		REGISTER_EVENT(GameEvent::AG_SET_PARAM,			"Sets the value of a component parameter. Param: AgentConfigParam(int,int/float/const char*) [second value ignored]")
		REGISTER_EVENT(GameEvent::AG_SET_ICON,			"Sets the name of the icon entity. Param: const char*")
		REGISTER_EVENT(GameEvent::AG_SET_ICONVISIBLE,		"Sets the visibility of the icon entity. Param: bool")
		REGISTER_EVENT(GameEvent::AG_SET_VISIBLE,			"Sets the visibility of the agent. Param: bool")
		
		REGISTER_EVENT(GameEvent::NV_SAY_SENTENCE,		"sentence to create speaking behavior for")
		REGISTER_EVENT(GameEvent::NV_HEAR_SENTENCE,		"sentence to create listening behavior for")
		REGISTER_EVENT(GameEvent::NV_HEAD_ANGLES,			"manual gaze target for the head (using Vec3f)")
		REGISTER_EVENT(GameEvent::NV_EYE_ANGLES,			"manual gaze target for the eyes (using Vec3f)")
		REGISTER_EVENT(GameEvent::EM_SET_EMOTION,			"set mood to a specific emotion, e.g. at the beginning of a dialogue (using string) ")
		REGISTER_EVENT(GameEvent::EM_OWN_EMOTION,			"emotional input for a character (using EmotionData)")
		//EM_OBSERVE_EMOTION,	"/// react to other characters' emotion (using EmotionData)")
		REGISTER_EVENT(GameEvent::EM_OWN_CONTEXT,			"add new context information to mood calculation(using EmotionData)")
		REGISTER_EVENT(GameEvent::D_INIT_SPEAKERS,		"initialize speakers ")
		REGISTER_EVENT(GameEvent::D_START_DIALOGUE,		"start playing the parsed dialogue")
		REGISTER_EVENT(GameEvent::CC_CHARACTER_ACTION,	"character performs an action, snt from BTcomp to CharControl")
		REGISTER_EVENT(GameEvent::CC_ABORT_ACTION,		"to abort an action that is performed")
		REGISTER_EVENT(GameEvent::CC_REPORT_COMPLETION,	"reports whether a character action was succesfull, or failed")
		REGISTER_EVENT(GameEvent::BT_ACTIVATE_ON_OFF,		"activates, or deactivates the BTComponent")
		
		REGISTER_EVENT(GameEvent::NM_CHANGED,				"navmesh changed")
		
		///////////////////////////////////////////////////////////////////////////////////////////
		/**
		 * 1. GET EVENTS:
		 * 
		 * @send:
		 * If these events are send by a component, it wants to get the current value of another component's attribute.
		 * 
		 * @listen:
		 * If a component listens to such an event, it has an attribut that it will return to the sender.
		 * 
		 * @eventData: Contains a pointer where the value should be copied to.
		 * 
		 * @allowed by:
		 * Every component can send such an event, only one should listen to it for setting the value.
		 * 
		 * @check?:
		 * You don't need to call checkEvent() before executeEvent().
		 */ 
		///////////////////////////////////////////////////////////////////////////////////////////
		REGISTER_EVENT(GameEvent::E_TRANSFORMATION,		"Copies the current object transformation to the memory buffer provided by the Event (currently float[16])")
		REGISTER_EVENT(GameEvent::E_MESH_DATA,			"Copies the mesh parameters of a mesh representation in Horde3D to the provided MeshData struct within the event")
		REGISTER_EVENT(GameEvent::E_GET_ACTIVE_CAM,		"Global event: get the currently active camera entity world id")
		REGISTER_EVENT(GameEvent::E_GET_PROPERTY,			"Gets a property")
		REGISTER_EVENT(GameEvent::E_GET_INTERACTION_POS,	"Gets the interaction position")
		REGISTER_EVENT(GameEvent::E_GET_SOUND_DISTANCE,	"Get the distance of the current sound node to the active listener. @data pointer to float")
		REGISTER_EVENT(GameEvent::E_GET_VISIBILITY,		"Returns whether the current entity is visible by the active cam")
		REGISTER_EVENT(GameEvent::E_GET_PROJECTION_MATRIX,"Returns the current camera projection matrix @data pointer to float[16] for the matrix")
		REGISTER_EVENT(GameEvent::E_GET_SCENEGRAPH_ID,	"Returns the entity's scenegraph id (hordeID)")
		REGISTER_EVENT(GameEvent::E_GET_ANIM_LENGTH,		"Get the length of an animation in seconds (using all frames and default speed).")
		REGISTER_EVENT(GameEvent::E_GET_SOCKET_DATA,		"Gets the data received by the socket component @data: pointer to char* where data is copied to")
		REGISTER_EVENT(GameEvent::E_GET_SOCKET_NEWEST_MSG,"Gets the newest message received by the socket component and throws away all older messages @data: pointer to char* where data is copied to")
		REGISTER_EVENT(GameEvent::EM_GET_MOOD_PAD,		"gets the current mood from the Emotion component as PAD values (using Vec3f)")
		REGISTER_EVENT(GameEvent::D_CURRENT_SENTENCE,		"get current sentence from Dialogue components")
		REGISTER_EVENT(GameEvent::IK_GETPARAMI,			"Gets an IK parameter (IK_Param) of type integer")
		REGISTER_EVENT(GameEvent::IK_GETPARAMF,			"Gets an IK parameter (IK_Param) of type float")
		REGISTER_EVENT(GameEvent::CC_CHECK_CONDITION,		"Gets sensory input for AI/BT")
		
		REGISTER_EVENT(GameEvent::AG_ANIM_GET_STATUS,		"Gets the animation's status. Param: AgentAnimationData(int), returns: int")
		REGISTER_EVENT(GameEvent::AG_ANIM_GET_SPEED,		"Gets the speed value of an animation. Param: AgentAnimationData(int), returns: float")
		REGISTER_EVENT(GameEvent::AG_ANIM_GET_EXTENT,		"Gets the spatial extent value of an animation. Param: AgentAnimationData(int), returns: float")
		REGISTER_EVENT(GameEvent::AG_ANIM_GET_STROKES,	"Gets the stroke repetition value of an animation. Param: AgentAnimationData(int), returns: int")
		REGISTER_EVENT(GameEvent::AG_MOVEMENT_GET_STATUS,	"Gets the movement's status. Param: AgentMovementData(int), returns: int")
		REGISTER_EVENT(GameEvent::AG_MOVEMENT_GET_SPEED,	"Gets the default movement speed. Param: AgentMovementData(), returns: float")
		REGISTER_EVENT(GameEvent::AG_GAZE_GET_STATUS,		"Gets the gaze node's status. Param: AgentGazeData(int), returns: int")
		REGISTER_EVENT(GameEvent::AG_GAZE_GET_SPEED,		"Gets the agent's default gaze speed. Param: AgentGazeData(), returns: float")
		REGISTER_EVENT(GameEvent::AG_FORMATION_GET_AGENTS,"Gets the members of the formation. Param: AgentFormationData(int**), returns: int")
		REGISTER_EVENT(GameEvent::AG_FORMATION_GET_TYPE,	"Gets the type of the formation. Param: AgentFormationData(), returns: int")
		REGISTER_EVENT(GameEvent::AG_FORMATION_GET_ENTRY,	"Computes, returns:  an entry point to the formation. Param: AgentFormationData(int), returns: Vec3f")
		REGISTER_EVENT(GameEvent::AG_GET_IPDIST,			"Gets the interpersonal distance constraints of the agent. Param: AgentFormationData(), returns: float,float")
		REGISTER_EVENT(GameEvent::AG_GET_ORIENTCUST,		"Gets the deviation value from the normal orientation of the agent. Param: AgentFormationData(), returns: float")
		REGISTER_EVENT(GameEvent::AG_GET_REPOSANIM,		"Gets the the repositioning animation name. Param: AgentFormationData(), returns: const char*")
		REGISTER_EVENT(GameEvent::AG_GET_PARAM,			"Gets the value of a component parameter. Param: AgentConfigData(), returns: float/int/const char*")
		
		///////////////////////////////////////////////////////////////////////////////////////////
		/**
		 * 1. CHANGE EVENTS:
		 * 
		 * @send/listen:
		 * These events inform that an attribute has changed, or an action has been executed
		 * 
		 * @eventData: Contains the new value for the attribute or optional information about the action.
		 * 
		 * @allowed by:
		 * Every component can listen to these events.
		 * Only one component should send them (with few exceptions).
		 * 
		 * @check?:
		 * You don't need to call checkEvent() before executeEvent().
		 */ 
		///////////////////////////////////////////////////////////////////////////////////////////
		REGISTER_EVENT(GameEvent::E_KEY_PRESS,			"A key has been pressed")
		REGISTER_EVENT(GameEvent::E_ANIM_STOPPED,			"Indicates that an animation has stopped")
		REGISTER_EVENT(GameEvent::E_SPEAKING_STOPPED,		"Speak a sentence is finished")
		REGISTER_EVENT(GameEvent::E_SPEAKING_STARTED,		"Speak a sentence is finished")
		REGISTER_EVENT(GameEvent::E_COLLISION,			"A collision occured")
		REGISTER_EVENT(GameEvent::E_ACTIVE_CAM_CHANGE,	"Global event: the current active camera has changed, @data: the cams worldID")
		REGISTER_EVENT(GameEvent::E_WITNESS_ACTION,		"Witness an action")
		REGISTER_EVENT(GameEvent::GB_KEY_PRESSED,			"Inidicates a key press event")
		REGISTER_EVENT(GameEvent::E_AILOD_CHANGE,			"Occurs when the ai lod of an entity has changed, @data: the entity's new lod value")
		REGISTER_EVENT(GameEvent::GL_DRAW,				"A GL function call. @data: Uses the DataType GLFunction. @allowed by: GLDrawingComponent listens, any componant may send")
		REGISTER_EVENT(GameEvent::GP_STATE_CHANGE,		"The state of a gamepad has changed")
		REGISTER_EVENT(GameEvent::NV_GAZING_START,		"informs agent that he is being gazed at")
		REGISTER_EVENT(GameEvent::NV_GAZING_STOP,			"informs agent that he is no longer being gazed at")
		REGISTER_EVENT(GameEvent::NV_SPEAKING_STOPPED,	"tells dialogue partner to stop listening ")
		REGISTER_EVENT(GameEvent::EM_MOOD_CHANGED,		"informs an agent that the emotion was updated (sending new mood as Vec3f)")
		REGISTER_EVENT(GameEvent::SP_RHEME_START,			"rheme started")
		REGISTER_EVENT(GameEvent::SP_THEME_START,			"theme started")
		REGISTER_EVENT(GameEvent::SP_SPOKEN_WORD,			"contains currently spoken word")
		REGISTER_EVENT(GameEvent::E_SOUND_STOPPED,		"the sound of the entity has stopped playing")
		REGISTER_EVENT(GameEvent::E_SOCKET_NEW_DATA,		"Sends all data received since the last update call")

		// TODO: move the following events to the right category
		REGISTER_EVENT(GameEvent::SP_BOOKMARK,"")
		REGISTER_EVENT(GameEvent::SP_TTS_PAUSE,"")
		REGISTER_EVENT(GameEvent::SP_TTS_RESUME,"")
		REGISTER_EVENT(GameEvent::SP_TTS_SKIP,"")			///Skip the next n sentences. If n is negative, the speech will skip backwards.

		//EVENTS FOR SUGARCANEISLAND PROJECT
		REGISTER_EVENT(GameEvent::SCI_NODE_ID,"")
		REGISTER_EVENT(GameEvent::SCI_DECISION_ID,"")
		REGISTER_EVENT(GameEvent::SCI_DECIDED_ID,"")
		REGISTER_EVENT(GameEvent::SCI_QUICKTIME_ID,"")
		REGISTER_EVENT(GameEvent::SCI_QUICKTIME_RESULT,"")
		REGISTER_EVENT(GameEvent::SCI_BLACKOUT,"")
		REGISTER_EVENT(GameEvent::SCI_END,"")
		REGISTER_EVENT(GameEvent::SCI_MODE,"")
		REGISTER_EVENT(GameEvent::SCI_ACTIONSOUND,"")
}

EventRegistry::~EventRegistry() {
	if (pData_ != 0x0) {
		delete pData_; 
		pData_ = 0x0;
	}
}

GameEvent::EventID EventRegistry::registerEvent(const char* aEventName, GameEvent::EventID id, const char* aEventDesc)
{
    std::map<unsigned long, GameEvent::EventID>::iterator it = pData_->nameIds_.find(hash(aEventName));
    if (it != pData_->nameIds_.end())
        return it->second;
	pData_->events_.insert(std::make_pair(id, EventData(aEventName, aEventDesc)));
	pData_->nameIds_.insert(std::make_pair(hash(aEventName), id));
	return id;
}

const char* EventRegistry::getEventName(GameEvent::EventID aID) const
{
	std::map<GameEvent::EventID, EventData>::iterator it = pData_->events_.find(aID);
	if (it != pData_->events_.end())
		return it->second.name_.c_str();
    return 0x0;
}

const char* EventRegistry::getEventDescription(GameEvent::EventID aID) const
{
	std::map<GameEvent::EventID, EventData>::iterator it = pData_->events_.find(aID);
	if (it != pData_->events_.end())
		return it->second.desc_.c_str();
    return 0x0;
}

GameEvent::EventID EventRegistry::getEventID(const char* aName) const
{
	std::map<unsigned long, GameEvent::EventID>::iterator it = pData_->nameIds_.find(hash(aName));
    if (it != pData_->nameIds_.end())
        return it->second;
	return GameEvent::E_INVALID;
}

void EventRegistry::dump(std::ostream* out, int type)
{
    if (!out)
        return;
    if (type == 1) {
        *out << "<html>\n\t<head>\n\t\t<title>Game Engine Event Information</title>\n\t</head>\n\t<body>\n";
        *out << "\t\t<h1>Game Engine Event Information</h1>\n";
        if (pData_->events_.empty()) {
            *out << "\t\t<p>No events</p>\n\t</body>\n</html>";
            return;
        }
        std::map<GameEvent::EventID, EventData>::const_iterator it = pData_->events_.begin();
        while (it != pData_->events_.end()) {
            *out << "\t\t<p><table border=\"0\">\n";
            *out << "\t\t\t<tr><td colspan=\"2\">";
            *out << "<b>Event " << it->first << ":</b> " << it->second.name_ << "</td></tr>\n";
            if (!it->second.desc_.empty()) {
                *out << "\t\t\t<tr><td>";
                *out << "<u>Description:</u></td><td>" << it->second.desc_ << "</td></tr>\n";
            }
            *out << "</td></tr>\n";

            *out << "\t\t</table></p>\n";
            ++it;
        }
        *out << "\n\t</body>\n</html>";
    } else if (type == 0) {
        *out << "\tGame Engine Event Information\r\n";
        *out << "====================================================\r\n\r\n";
        if (pData_->events_.empty()) {
            *out << "No events";
            return;
        }
        std::map<GameEvent::EventID, EventData>::const_iterator it = pData_->events_.begin();
        while (it != pData_->events_.end()) {
            *out << "Event " << it->first << ": " << it->second.name_ << "\r\n";
            if (!it->second.desc_.empty())
                *out << "\tDescription: " << it->second.desc_ << "\r\n";
            *out << "\r\n";
            ++it;
        }
    } else if (type == 2) {
        *out << "<EventTable>\n";
        std::map<GameEvent::EventID, EventData>::const_iterator it = pData_->events_.begin();
        while (it != pData_->events_.end()) {
            *out << "\t<Event id=\"" << it->first << "\" name=\"" << it->second.name_ << "\">\n";
            if (!it->second.desc_.empty()) {
                *out << "\t\t<Description>" << it->second.desc_ << "</Description>\n";
            }
            "\t</Event>\n";
            ++it;
        }
    }
}

size_t EventRegistry::eventCount() const {
	return pData_->events_.size();
}